# Copyright (C) 2008 Wes Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Display::Bounds;

use strict;
use GeoDB::Display;
use GeoDB::Utils;

our @ISA = qw(GeoDB::Display);

our $VERSION = "1.11";

sub display {
     my ($self, $set, $dbh) = @_;

     GEODEBUG(3, "Displaying set boundaries\n");
     my @bounds = $set->get_bounds();
     my $latave = ($bounds[0]+$bounds[2])/2;
     my $lonave = ($bounds[1]+$bounds[3])/2;
     return "min    lat: $bounds[0]\nmax    lat: $bounds[2]\n" .
       "min    lon: $bounds[1]\nmax    lon: $bounds[3]\n" .
	 "center lat: $latave\ncenter lon: $lonave\n";
}


1;

=head1 NAME

Bounds - Show the boundary latitude/longitude of a set.

=head1 DESCRIPTION

START HELP

  Example 1:
  geoqo -s set:import:minneapolis -d bounds
  min    lat: 44.888483
  max    lat: 45.061767
  min    lon: -93.391317
  max    lon: -93.146517
  center lat: 44.975125
  center lon: -93.268917

The Bounds display shows the maximum and minimum latitude and
longitude found in the set.  It also outputs the center of the bounds
as well.

END HELP

=cut

