# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Display::Graph::Find;

use strict;
use GeoDB::Display;
use GeoDB::Utils;
use GeoDB::Display::Graph;
my $have_date_parse = eval "require Date::Parse";

our @ISA = qw(GeoDB::Display::Graph);

our $VERSION = "1.11";

our @GEOOPTIONS = 
  (
   { name => 'finder',
     help => 'Sets the NAME of the finder you\'re looking for to a particular geocaching name (ie, the name of the user that is logging the finds).  99/100 egoists recommend using your own geocaching name here.' },
   { name => 'logtype',
     help => 'Defines the log type to graph.',
     default => 'Found it' }
  );

sub get_date {
    my @a = localtime($_[0]);
    my $date = sprintf("%04d/%02d/%02d", $a[5] + 1900, $a[4]+1, $a[3]);
    my $time = sprintf("%02d:%02d:%02d", $a[2], $a[1], $a[0]);
    return "$date $time";
}

sub get_graph_options {
    my $self = shift;
    return
      [ x_label => 'time',
	y_label => 'Finds',
	title => "Finds by $self->{options}{finder}",
	'x_number_format' => \&get_date,
	'x_ticks', 'staggered',
	'x_tick_number', 'auto',
	'x_labels_vertical', '1',
#	'-charttype', 'Pareto',
	'legend', [],

      ];
}

sub get_graph_data {
    my ($self, $set, $dbh) = @_;
    my (@data);

    my $name = $self->{'options'}{'finder'};
    my $logtype = $self->{'options'}{'logtype'};

    GEOERR("Can't display Find graphs: The Date::Parse module is needed.\n")
      if (!$have_date_parse);

    GEOERR("Graph/Find requires a finder= parameter\n") if (!$name); 

    # collect the day/log entry by a particular person ($name)
    GEODEBUG(3, "Graphing finds: name=$name, num=" . ($set->num()) . "\n");
    $set->load({logfinder => $name});
    $set->foreach(sub {
		      foreach my $logid (keys(%{$_[0]->{'logs'}})) {
			  my $log = $_[0]->{'logs'}{$logid};
			  if ($log->{'finder'} eq $name &&
			      $log->{'type'} eq $logtype) {
			      push @data, $log->{'date'};
			      last;
			  }
		      }
		  });

    @data = sort @data;
    my $count = 1;
    map { $_ = [Date::Parse::str2time($_), $count++]; } @data;

    return \@data;
}

1;

=head1 NAME

graph/find - Displays a graph of the find rate for a given user

=head1 DESCRIPTION

START HELP

  Example:
  geoqo -s log:finder=Yamar -d graph/find:finder=Yamar

This shows a graph of finds over time by a given person.  Generally,
you'll want to run this after you've incorporated a pocket query of all
of your finds.  You can run it against other user names too, but it's
unlikely you have every log record for them unless you've imported
their find pocket query too.

END HELP

=head3 Options:

=over

=item finder=NAME

Sets the NAME of the finder you're looking for to a particular
geocaching name (ie, the name of the user that is logging the finds).
99/100 egoists recommend using your own name here.

=back

=cut
