# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Display::Graph::Placed;

use strict;
use GeoDB::Display;
use GeoDB::Utils;
use GeoDB::Display::Graph;
my $have_date_parse = eval "require Date::Parse";

our @ISA = qw(GeoDB::Display::Graph);

our $VERSION = "1.11";

our %tokens = 
  (month => 4,
   day => 3,
   year => 5,
   weekday => 6,
   yearday => 7);
my @timelabels = keys(%tokens);

our @GEOOPTIONS = 
  (
   { name => 'type',
     type => 'menu',
     values => \@timelabels,
     help => 'Examines creation date based on this frequency (X-axis column).  PERIOD may be either month, day, year, weekday or yearday.  The default is month (ie, the graph will be from 1-12 showing the number of caches placed during each month in your loaded search set).',
     default => 'month' },
  );

sub get_graph_options {
    my $self = shift;
    return
      [ y_label => 'Number Placed',
	title => "Caches Placed by $self->{options}{type}",
	'-charttype', 'Bars',
	'already_in_bins' => 1,
      ];
}

sub get_graph_data {
    my ($self, $set, $dbh) = @_;
    my (@data);

    GEOERR("Can't display Find graphs: The Date::Parse module is needed.\n")
      if (!$have_date_parse);

    my $type = $self->{'options'}{'type'} || 'month';
    my $fieldnum = $tokens{$type};
    die("unknown type: $type") if (!$fieldnum);

    # collect the day/log entry by a particular person ($name)
    GEODEBUG(3, "Graphing placement by $type (caches: " . ($set->num()) .")\n");
    $set->load_parts();
    my (@thecount);
    $set->foreach(sub {
		      my $unixtime = 
			Date::Parse::str2time($_[0]->{'data'}{'time'});
		      my @x = localtime($unixtime);
		      $thecount[$x[$fieldnum]]++;
		  });

    for (my $i = 0; $i <= $#thecount; $i++) {
	push @data, [$i+1, $thecount[$i]];
    }

    return \@data;
}

1;

=head1 NAME

graph/placed - Display a graph of number of caches placed by month

=head1 DESCRIPTION

START HELP

  Example:
  geoqo -s any: -d graph/placed:type=month

Shows a graph displaying which during months, weeks, etc the caches
were placed.  Really this just gives an indication of which months are
the most popular months for placing a cache (probably due to weather).

END HELP

=cut
