# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Display::Gui::List::Plugin::Rate;

use strict;

use GeoDB::Display::Gui::List::Plugin;
use QWizard;
use QWizard::API;
use GeoDB::Set;
use GeoDB::Utils;

our @ISA = qw(GeoDB::Display::Gui::List::Plugin);

our $VERSION = "1.11";

our @GEOOPTIONS =
  (
   { name => 'ratings',
     title => 'Ratings to create menus for:',
     type => "textbox",
     default => "overall\nfun\nreturnability\nrecommendabiltiy\nlocation",
   },
   { name => 'doothers',
     type => 'checkbox',
     title => 'Include menus for rating names other people have set:',
     indent => 1,
     default => 1,
   },
   { name => 'fetchfirst',
     type => 'checkbox',
     title => 'Fetch other ratings from the rating server first:',
     default => 1,
   },
   { name => 'publish',
     type => 'checkbox',
     title => 'Publish my ratings to the rating server:',
     default => 1,
   },
   { name => 'rated',
     type => 'checkbox',
     title => 'Tag these caches as \'rated\':',
     helpdesc => '(can be skiped in future runs)',
     default => 1,
   }
  );

sub get_ratings {
    my ($self) = @_;
    # allows the list to be newline/space separated or , separated
    my @ratings;
    if ($self->{'options'}{'ratings'} =~ /,/) {
	@ratings = split(/,\s*/,$self->{'options'}{'ratings'});
    } else {
	@ratings = split(/\n\s*/,$self->{'options'}{'ratings'});
    }
    return @ratings;
}

sub get_early_headers {
    my ($self, $set) = @_;
    if ($self->{'options'}{'fetchfirst'}) {
	GEODEBUG(4,"prefetching ratings\n");
	my $master = $self->get_master();
	$master->modify($set, "rate:fetch=1");
    }
    return;
}

sub get_extra_rows {
    my ($self, $cache, $num) = @_;
    my $row = ['', 'Ratings:'];
    my @ratings = $self->get_ratings();
    $cache->load({nologs => 1, noattributes => 1});
    push @ratings, keys(%{$cache->{'ratings'}});

    # weed out duplicates
    my %ratings = map {$_,$_} @ratings;
    delete $ratings{'ratingged'}; # delete internal
    @ratings = keys(%ratings);

    my @rows;
    foreach my $rating (@ratings) {
	my $label = $rating;
	$label .= "($cache->{'ratings'}{$rating}{'count'})"
	  if (exists($cache->{'ratings'}{$rating}) &&
	      $cache->{'ratings'}{$rating}{'count'});

	# not efficient to do this for each cache shown
	my $labels = ['', $label];
	foreach my $rate (1..10) {
	    push @$labels, $rate, "$label: $rate";
	}

	push @$row, { type => 'menu',
		      name => $cache->{'data'}{'ident'} . ':' . $rating,
		      labels => $labels,
		      default => ((exists($cache->{'ratings'}{$rating}) &&
				   exists($cache->{'ratings'}{$rating}{'value'})) ? 
				  $cache->{'ratings'}{$rating}{'value'} : '')
		    };
	if ($#$row > 4) {
	    push @rows, $row;
	    $row = ['',''];
	}
    }
    push @rows, $row if ($#$row != 1);
    return @rows;
}

sub post_answers {
    my ($self, $qw, $set, $listobj) = @_;

    my @ratings = $self->get_ratings();
    $set->foreach(sub {
		      my ($cache) = @_;
		      foreach my $rating (@ratings) {
			  if (qwparam($cache->{'data'}{'ident'} . ":$rating")) {
			      $cache->set_rating($rating, qwparam($cache->{'data'}{'ident'} . ":$rating"));
			  } else {
			      $cache->remove_rating($rating);
			  }
		      }
		      if ($self->{'options'}{'rated'}) {
			  $cache->set_tag('rated');
		      }
		  });
    if ($self->{'options'}{'publish'}) {
	my $master = $self->get_master();
	$master->modify($set, "rate:publish=1");
    }
    return 0;
}

1;
