# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Display::Gui::List::Plugin::Tag;

use strict;

use GeoDB::Display::Gui::List::Plugin;
use QWizard;
use QWizard::API;
use GeoDB::Set;
use GeoDB::Utils;

our @ISA = qw(GeoDB::Display::Gui::List::Plugin);

our $VERSION = "1.11";

our @GEOOPTIONS =
  (
   { name => 'tags',
     title => 'Tags to create buttons for:',
     type => "textbox",
     default => "nicecontainer\ngoodforkids\nprettyarea\ncito\nhike\nunique\nfun\nhistorical",
   },
   { name => 'doothers',
     type => 'checkbox',
     title => 'Include buttons for tags other people have set:',
     indent => 1,
     default => 1,
   },
   { name => 'fetchfirst',
     type => 'checkbox',
     title => 'Fetch other tags from the tag server first:',
     default => 1,
   },
   { name => 'publish',
     type => 'checkbox',
     title => 'Publish my tags to the tag server:',
     default => 1,
   },
   { name => 'didtagged',
     type => 'checkbox',
     title => 'Mark these caches as done being tagged:',
     helpdesc => '(so you can skip them in future runs)',
     default => 1,
   }
  );

sub get_early_headers {
    my ($self, $set) = @_;
    if ($self->{'options'}{'fetchfirst'}) {
	GEODEBUG(4,"prefetching tags\n");
	my $master = $self->get_master();
	$master->modify($set, "tag:fetch=1");
    }
    return;
}

sub get_tags {
    my ($self) = @_;
    # allows the list to be newline/space separated or , separated
    my @tags;
    if ($self->{'options'}{'tags'} =~ /,/) {
	@tags = split(/,\s*/,$self->{'options'}{'tags'});
    } else {
	@tags = split(/\n\s*/,$self->{'options'}{'tags'});
    }
    return @tags;
}

sub get_extra_rows {
    my ($self, $cache, $num) = @_;
    my $row = ['', 'Tags:'];
    my @tags = $self->get_tags();
    $cache->load({nologs => 1, noattributes => 1});
    push @tags, keys(%{$cache->{'tags'}});

    # weed out duplicates
    my %tags = map {$_,$_} @tags;
    delete $tags{'tagged'}; # delete internal
    delete $tags{'rated'}; # delete internal
    @tags = keys(%tags);

    my @rows;
    foreach my $tag (@tags) {
	my $label = $tag;
	$label .= "($cache->{'tags'}{$tag}{'count'})"
	  if (exists($cache->{'tags'}{$tag}) &&
	      $cache->{'tags'}{$tag}{'count'});
	push @$row, { type => 'checkbox',
		      name => $cache->{'data'}{'ident'} . ':' . $tag,
		      button_label => $label,
		      values => [1,0],
		      default => ((exists($cache->{'tags'}{$tag}) &&
				   $cache->{'tags'}{$tag}{'ismine'}) ? 
				  1 : 0)
		    };
	if ($#$row > 4) {
	    push @rows, $row;
	    $row = ['',''];
	}
    }
    push @rows, $row if ($#$row != 1);
    return @rows;
}

sub post_answers {
    my ($self, $qw, $set, $listobj) = @_;

    my @tags = $self->get_tags();
    $set->foreach(sub {
		      my ($cache) = @_;
		      foreach my $tag (@tags) {
			  if (qwparam($cache->{'data'}{'ident'} . ":$tag")) {
			      $cache->set_tag($tag);
			  } else {
			      $cache->remove_tag($tag);
			  }
		      }
		      if ($self->{'options'}{'didtagged'}) {
			  $cache->set_tag('tagged');
		      }
		  });
    if ($self->{'options'}{'publish'}) {
	my $master = $self->get_master();
	$master->modify($set, "tag:publish=1");
    }
    return 0;
}

1;
