# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Display::Help;

use strict;
use GeoDB::Display;
use GeoDB::Utils;

maybe_load('IO::String',"The IO::String perl module is required for online help support.\n");
maybe_load('Pod::Text',"The Pod::Text perl module is required for online help support.\n");

our @ISA = qw(GeoDB::Display);

our $VERSION = "1.11";

our @GEOOPTIONS = 
  (
   { name => 'topic',
     help => 'Specifies the topic to display help on.' },
  );

sub display {
     my ($self, $set, $dbh) = @_;

     if (!$self->{'options'}{'topic'}) {
	 print "GeoQO/GeoDB help system:

Please use one of the following help terms for more information about
particular sub-topics of help:

  geoqo -d help:topic=I<TOPIC/SUBTOPIC>

Where I<TOPIC> is one of:

  import
  search
  modify
  display
  export

You should also pick a particular subtopic to list as well if you want
to see help on a specific subtopic.  As an example, to display help on
the I<List> output for displaying of waypoints specify:

  geoqo -d help:display/list

The parent topics should provide a list of each of the available
subtopics for their category.

";
	 return;
     }

     my $output;
     if ($self->{'options'}{'topic'}) {
	 my $name = $self->{'options'}{'topic'};
	 $name =~ s/^(.)(.*)/uc($1) . lc($2)/e;
	 $name =~ s/\/(.)([^\/]*)/"\/" . uc($1) . lc($2)/ge;
	 foreach my $dir (@INC) {
	     if (-f "$dir/GeoDB/$name.pm") {
		 my $parser = new Pod::Text();
		 my $str = IO::String->new();
		 $parser->parse_from_file("$dir/GeoDB/$name.pm", $str);
		 my $ref = ${$str->string_ref()};
		 $ref =~ s/.*(START|BEGIN) *HELP[ \t]*\n//s;
		 $ref =~ s/END *HELP.*//s;
		 $output .= $ref,"\n";

		 # grab the options from the module
		 $name =~ s/\//::/g;
		 $name = "GeoDB::$name";
		 my $haveit = eval "require ${name}";
		 if ($haveit) {
		     my @ops = eval "\@${name}::GEOOPTIONS";
		     if ($#ops > -1) {
			 $output .= "Options:  \n";
			 foreach my $o (@ops) {
			     next if (ref($o) ne 'HASH');
			     $output .= "  $o->{name}:\n";
			     $output .= maybe_wrap($o->{'title'}, "    ")
			       if ($o->{'title'});
			     $output .= maybe_wrap($o->{'text'}, "    ")
			       if ($o->{'text'});
			     $output .= maybe_wrap($o->{'helpdesc'}, "      ")
			       if ($o->{'helpdesc'});
			     $output .= "\n"
			       if ($o->{'helpdesc'} || $o->{'title'});
			     $output .= maybe_wrap($o->{'help'}, "    "),"\n"
			       if ($o->{'help'});
			     $output .= "    default: $o->{default}\n"
			       if(defined($o->{'default'}));
			     $output .= "\n";
			 }
		     }
		 }
		 return $output;
	     }
	     GEODEBUG(3,"not in here: $dir/GeoDB/$name.pm\n");
	 }
     }
     return $output;
}


1;

=head1 NAME

Help - Display Help Information

=head2 DESCRIPTION

START HELP

The help module displays help on a given topic.  You can specify help
on a given topic.

  Example 1:
  geoqo -d help

  Example 2:
  geoqo -d help:topicname

The help routine displays help for the various geoqo plugins.

END HELP

=cut

