# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Export::Density;

use strict;
use GeoDB::Utils;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(hsv_to_rgb);

sub get_density_map {
    my ($self, $set) = @_;

    # width/height of the image
    my $binnumx =
      $self->{'options'}{'width'} || $self->{'options'}{'size'} || 200;
    my $binnumy =
      $self->{'options'}{'height'} || $self->{'options'}{'size'} || 200;

    # radius not diameter
    my $spread = $self->{'options'}{'spread'} || 5;

    my ($maxlat, $minlat, $maxlon, $minlon) = (-10000, 10000, -10000, 10000);
    $set->foreach(sub {
		      $maxlat = max($_[0]{'data'}->{'lat'}, $maxlat);
		      $minlat = min($_[0]{'data'}->{'lat'}, $minlat);
		      $maxlon = max($_[0]{'data'}->{'lon'}, $maxlon);
		      $minlon = min($_[0]{'data'}->{'lon'}, $minlon);
		  });

    # let user override certain ones
    $maxlat = $self->{'options'}{'nmax'} || $maxlat;
    $minlat = $self->{'options'}{'nmin'} || $minlat;
    $maxlon = $self->{'options'}{'wmax'} || $maxlon;
    $minlon = $self->{'options'}{'wmin'} || $minlon;

    ($maxlat, $maxlon) = parse_coords($maxlat, $maxlon);
    ($minlat, $minlon) = parse_coords($minlat, $minlon);

    GEODEBUG(4,"boundaries: $maxlat, $minlat, $maxlon, $minlon\n");
    my $latdiff = $maxlat - $minlat;
    my $londiff = $maxlon - $minlon;


    GEODEBUG(5,"filling circle\n");
    my $textcir;
    my @spread;
    for (my $i = 0; $i <= $spread*2; $i++) {
	for (my $j = 0; $j <= $spread*2; $j++) {
	    my $xd = $spread-$i;
	    my $yd = $spread-$j;
	    $spread[$i][$j] = (sqrt($xd*$xd + $yd*$yd) <= $spread) ? 1 : 0;
	    if ($spread[$i][$j]) {
		$textcir .= "X";
	    } else {
		$textcir .= ".";
	    }
	}
	$textcir .= "\n";
    }
    GEODEBUG(7,"circle:\n$textcir");

    GEODEBUG(5, "putting caches into bin buckets\n");

    my ($x, $y, @results);

    my $count = 0;
    my $next = 0;
    $self->setup_progress($set->num(), "Creating Density: analyzing waypoints");
    $set->foreach(
		  sub {
		      $next = $self->set_progress($count++)
			if ($count >= $next);
		      $x = int(($_[0]{'data'}{'lat'} - $minlat) * $binnumx / $latdiff);
		      $y = int(($_[0]{'data'}{'lon'} - $minlon) * $binnumy / $londiff);
		      $results[$x][$y]++;
		  });
    $self->stop_progress();

    my @spreadresults;
    my $maxcount = 0;

    $next = 0;
    $self->setup_progress($binnumx, "Creating Density: Spreading Results");
    for (my $i = 0; $i <= $binnumx; $i++) {
	$next = $self->set_progress($i);
	for (my $j = 0; $j <= $binnumy; $j++) {
	    for (my $x = $i-$spread; $x <= $i+$spread; $x++) {
		for (my $y = $j-$spread; $y <= $j+$spread; $y++) {
		    if ($spread[$x - $i + $spread][$y - $j + $spread]) {
			if ($x >= 0 && $x <= $#results
			    && $y >= 0 && $y <= $#results) {
			    $spreadresults[$i][$j] += $results[$x][$y];
			}
		    }
		}
	    }
	    $maxcount = max($maxcount,$spreadresults[$i][$j])
	}
    }
    $self->stop_progress();

    return (\@spreadresults, $maxcount,
	    $maxlat, $minlat, $maxlon, $minlon);
}

sub hsv_to_rgb {
    my $h = shift;
    $h = $h * 6;
    my $s = 1;
    my $v = 1;
    my $i = int($h);
    my $f = $h - $i;
    if ($i % 2 == 0) {
	$f = 1 - $f;
    }
    my $m = $v * (1 - $s);
    my $n = $v * (1 - $s * $f);
    return ($v * 255, $n * 255, $m * 255) if ($i == 6 || $i == 0);
    return ($n * 255, $v * 255, $m * 255) if ($i == 1);
    return ($m * 255, $v * 255, $n * 255) if ($i == 2);
    return ($m * 255, $n * 255, $v * 255) if ($i == 3);
    return ($n * 255, $m * 255, $v * 255) if ($i == 4);
    return ($v * 255, $m * 255, $n * 255) if ($i == 5);
}

1;
