# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Modify::Only;

use strict;
use GeoDB::Modify;

our @ISA = qw(GeoDB::Modify);
our $VERSION = "1.11";

sub modify {
    my ($self, $cache, $set) = @_;

    $cache->load_parts({ attributes => 1});
    foreach my $opt (@{$self->{'optionsarray'}}) {
	my $name = $opt->[0];
	my $type = 'data';
	$type = 'attributes' if ($name =~ s/^attr://);
	if ($opt->[2] eq '<>') {
	    if ($cache->{$type}{$name} eq $opt->[1]) {
		push @{$self->{'dropidents'}}, $cache->{'data'}{'ident'};
	    }
	}
	elsif ($opt->[2] eq '=') {
	    if ($cache->{$type}{$name} ne $opt->[1]) {
		push @{$self->{'dropidents'}}, $cache->{'data'}{'ident'};
	    }
	}
	elsif ($opt->[2] eq '=-') {
	    if (exists($cache->{$type}{$name})) {
		push @{$self->{'dropidents'}}, $cache->{'data'}{'ident'};
	    }
	}
	elsif ($opt->[2] eq '=!') {
	    if (!exists($cache->{$type}{$name})) {
		push @{$self->{'dropidents'}}, $cache->{'data'}{'ident'};
	    }
	}
    }
}

sub clean_up {
    my ($self, $set) = @_;
    foreach my $ident (@{$self->{'dropidents'}}) {
	$set->remove_by_ident($ident);
    }
}

1;

=head1 NAME

Only - Only keeps caches from the set based on criteria

=head1 DESCRIPTION

START HELP

This is most useful when doing really complex exporting.  See Yamar's
usage description on the web page for why this is helpful sometimes.
It's mostly useful when you want to only include certain waypoints in
a given export file, and you want to create lots of export files
(TomTom's require one file per POI you wish to create, and I (Yamar)
create different POIs for each cache type, size, etc.)

The argument formatting is:

  only:field=value

where field is the field (or attribute prefixed with 'attr:') name,
and value is the exact value.  The valid match types are:

=over

=item only:field=value

Includes the waypoint only if the value is exactly the same as the
waypoint's field value.

=item only:field<>value

Matches if the field B<is not> that value

=item only:field=-

Matchs if the field doesn't exist.

=item only:field=+

Same as =- but only includes the waypoint if it does exist.

=back

END HELP

=cut
