# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Modify::Solved;

use strict;
use GeoDB::Modify;
use GeoDB::Utils;

our @ISA = qw(GeoDB::Modify);

our $VERSION = "1.11";

our @GEOOPTIONS = 
  (
   { name => 'nattr',
     help => 'The name of the attribute storing the new north coordinate',
     default => 'puzn'},
   { name => 'wattr',
     help => 'The name of the attribute storing the new west coordinate',
     default => 'puzw'},
   { name => 'dropunparsed',
     type => 'checkbox',
     help => 'Drops caches that do not have puzzle answers but are not parseable',
     default => 1
   },
  );


sub modify {
    my ($self, $cache) = @_;

    $cache->load_parts({attributes => 1});
    my $n = $self->{'options'}{'nattr'};
    my $w = $self->{'options'}{'wattr'};


    if ($cache->{'attributes'}{$n} ||
	$cache->{'attributes'}{$w}) {
	my ($nc, $wc) = parse_coords($cache->get_attribute($n),
				     $cache->get_attribute($w));
	if (!$nc || !$wc) {
	    GEOWARN("Failed to parse $cache->{data}{ident} solution coordinates.   Either attribute $self->{'options'}{'nattr'} = \"$cache->{'attributes'}{$n}\" or $self->{'options'}{'wattr'} = \"$cache->{'attributes'}{$w}\" isn't understandable");
	    push @{$self->{'dropidents'}}, $cache->{'data'}{'ident'}
	      if ($self->{'options'}{'dropunparsed'});
	    return;
	}
	$cache->{'data'}{'lat'} = $nc;
	$cache->{'data'}{'lon'} = $wc;
    }
}

sub clean_up {
    my ($self, $set) = @_;
    foreach my $ident (@{$self->{'dropidents'}}) {
	$set->remove_by_ident($ident);
    }
}

1;

=head1 NAME

Solved - Modifes the lat/lon coordinates to reflect solved puzzle coordinates

=head1 DESCRIPTION

START HELP

Example of solving a puzzle and saving it:

  > geoqo -s ident:GCABCD -m 'setattr:puzn=N38 42.444,puzw=W121 42.444' --save

Example of exporting it with the replaced coordinates:

  > geoqo -s ident:GCABCD -e 'solved:+loc:out.loc'

This plugin replaces the latitude and longitude coordinates of a
waypoint with those in the puzn and puzw attributes.  Use the above
examples as a guide for solving a puzzle and then using the solved:
plugin when exporting your database ready for uploading into a GPS.

END HELP

=cut
