# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Search::Attr;

=head1 NAME

Attr - search waypoint attributes

=head1 SYNOPSIS

geoqo -s attr:myattr=myvalue -l

=head1 DESCRIPTION

START HELP

Attr searches attributes that you yourself have placed on a cache
using the modify I<setattr> module.

  > geoqo -s ident:gcabcd -m setattr:found=1 --save

  > geoqo -s attr:found=1 -d count
  Search Result Count: 1

NOTE: this does NOT search groundpseak (geocaching.com) attributes.
For this you should use the attribute: search plugin intstead.

END HELP

=head1 AUTHOR

Wes Hardaker (AKA Yamar) <hardaker ATAT users.sourceforge.net> 

=cut

use GeoDB::Utils;

use strict;

our @ISA = qw(GeoDB::Search);

our $VERSION = "1.11";

sub search {
    my ($self, $criteria, $type) = @_;

    my $clause = "";
    my @values;

    if ($self->{'optionsarray'}[0][2] eq '=-') {
	GEODEBUG(4, "attr query: treating as not exists");

	# XXX: broken for multiple queries.
	foreach my $opt (@{$self->{'optionsarray'}}) {
	    $clause = "waypoints.ident not in 
                 (select ident from waypointdata where attrname = ?)";
	    push @values, $opt->[0];
	}
	return ($clause, \@values);
    }

    foreach my $opt (@{$self->{'optionsarray'}}) {
	$clause .=
	  GeoDB::Search::get_search_subexpression("or ( attrname ARG ? ",
						  '==', $opt->[0],
						  \@values);
	$clause .=
	  GeoDB::Search::get_search_subexpression(" and attrvalue ARG ? )",
						  $opt->[2], $opt->[1],
						  \@values);
    }
    $clause =~ s/^or //;

    $clause =
      "waypoints.ident in (select ident from waypointdata where $clause)";

    GEODEBUG(4,"attr query: $clause\n");

    return  ($clause, \@values);
}

1;
