# Copyright (C) 2007-2009 Hardaker
# License: GNU GPLv2.  See the COPYING file for details.
package GeoDB::Waypoint::Wigle;

use strict;
use GeoDB::Utils;
use GeoDB::Waypoint;

our $VERSION = "1.11";
our @ISA=qw(GeoDB::Waypoint);

# database handles
my ($getcacheh, $getidh, $newwayidh, $updatewh,
    $logdelh, $logaddh, $logselh, $logselfh);

# 'ident' not listed intentionally
our @wigle_dbcols = qw(
		       ssid
		       netid
		       discover
		       channel
		       wigtype
		       freenet
		       firsttime
		       wep
		       comment
		       qos
		       lastupdt
		       paynet
		       userfound
		      );

# 'ident' not listed intentionally
our @wigle_class_cols = qw(
			   wigle_ssid
			   wigle_netid
			   wigle_discover
			   wigle_channel
			   wigle_type
			   wigle_freenet
			   wigle_firsttime
			   wigle_sep
			   wigle_comment
			   wigle_qos
			   wigle_lastupdt
			   wigle_paynet
			   wigle_userfound
			  );

our @wigle_waypoint_cols = (
			       @GeoDB::Waypoint::waypoints_dbcols,
			       @wigle_dbcols
			      );

sub init {
    my $dbh = shift;

    return if (!$dbh);

    # wigle handles
    ($getidh, $newwayidh, $updatewh) = 
      create_waypoint_handles($dbh, 'wigles', \@wigle_dbcols);

    GeoDB::Waypoint::register_waypoint_type('Wigle',
					    'GeoDB::Waypoint::Wigle',
					    'wigles',
					    \@wigle_dbcols,
					    \@wigle_class_cols);

    return 1;
}

# if it already exists in the db, return 1
sub typedata_exists {
    my ($self) = @_;
    $getidh->execute($self->{'data'}{'ident'});
    my $row = $getidh->fetchrow_arrayref();
    $getidh->finish();
    return 1 if $row;
    return 0;
}

# save the current cache to the DB (either update or insert a new row)
sub save {
    my $self = shift;
    my $modgh = $newwayidh;
    # save to db

    $self->SUPER::save(@_);

    if ($self->typedata_exists()) {
	GEODEBUG(6,"updating wigle $self->{data}{ident}\n");
	$modgh = $updatewh;
    } else {
	GEODEBUG(6,"inserting wigle $self->{data}{ident}\n");
    }
    my $data = $self->{'data'};

    $modgh->execute(
		    $data->{'wigle_ssid'},
		    $data->{'wigle_netid'},
		    $data->{'wigle_discover'},
		    $data->{'wigle_channel'},
		    $data->{'wigle_type'},
		    $data->{'wigle_freenet'},
		    $data->{'wigle_firsttime'},
		    $data->{'wigle_sep'},
		    $data->{'wigle_comment'},
		    $data->{'wigle_qos'},
		    $data->{'wigle_lastupdt'},
		    $data->{'wigle_paynet'},
		    $data->{'wigle_userfound'},
		    $data->{'ident'},
		  );
    if ($modgh->errstr) {
	GEOERR("Error saving $data->{ident}: " . $modgh->errstr . "\n");
    }
}

sub ssummary {
    my ($self, $count, $options) = @_;

    return $self->sprint_summary_line($count, ($options->{'format'} ? "$options->{'format'}\n" : "%{3:count} %{8:ident} %{66:ssid}\n"));
}

sub sdisplay {
    my ($self, $dologs) = @_;
    my $data = $self->{'data'};
    my $out;
    $out .= sprintf("%-8s   %-31s Location:  %s, %s\n",
	   $data->{'ident'}, $data->{'type'} . " / " . $data->{'subtype'},
	   sprint_coords($data->{'lat'}, $data->{'lon'}));
    $out .= sprintf("SSID:      %s\n", $self->{'data'}{'ssid'});
    my $time = $data->{'firsttime'};
    $time =~ s/T.*//;
    $out .= sprintf("Created:   %s\n", $time,);
    $out .= sprintf("Freenet:   %s\n", $data->{'wigle_freenet'});
    $out .= sprintf("WEP:       %s\n", $data->{'wigle_wep'});
    $out .= sprintf("Comment:   %s\n", $data->{'wigle_comment'})
      if ($data->{'wigle_comment'});
    $out .= $self->sshow_attributes();
    $out .= $self->sshow_tags();
    $out .= $self->sshow_ratings();
    return $out;
}

sub remove_from_specific_tables {
    my $self = shift;
    $self->remove_from_a_table_by_ident('wigles');
}

1;
